/*
 *	emu.c
 *
 *	ATARI VCS/EMU
 *
 *	* Tue Mar 15 20:31:00 JST 2005 Naoyuki Sawa
 *	- 쐬JnB
 */
#include "app.h"

/****************************************************************************
 *	
 ****************************************************************************/

int
vcs_init(const char* fname)
{
	/* ROMt@C̃tbVAhX擾܂B */
	stream_stop();  /* ĂȂ͂ǁAÔߊmɒ~ */
	ldirect_free(); /* ߂ȂĂvǁAÔ */
	rom = mmap(fname, &rom_size, 1/*DEFRAG*/);
	ldirect_init(LDIRECT_INTERVAL);
	if(!rom) goto L_ERR;
	if(rom_size != 0x1000) goto L_ERR;

	/* ROMC[WRs[܂B */
	memcpy(&m6502mem[0xf000], rom, 0x1000);

	/* TCNJE^NA܂B */
	CYCLE = 0;
	NOW   = 0;

	/* TIAAPIAZbg܂B */
	tia_reset(tbuff, NOW);
	pia_reset(NOW);

	/* CPUZbg܂B */
	m6502cpu_reset(vcs_read, vcs_write);

	return 0;

L_ERR:
	return -1;
}

void
input_update()
{
	int pad = pcePadGetDirect();
	//
	int i4;
	int i5;

	i4 = (pad & PAD_A) ? 0/*On=Lo*/ : 1/*Off=Hi*/;
	i5 = (pad & PAD_B) ? 0/*On=Lo*/ : 1/*Off=Hi*/;
	tia_set_i4_i5(i4, i5);

	pia.SWCHA = 0xff;
	if(pad & PAD_RI) pia.SWCHA &= ~(1<<7);		/* D7: P0 right */
	if(pad & PAD_LF) pia.SWCHA &= ~(1<<6);		/* D6: P0 left  */
	if(pad & PAD_DN) pia.SWCHA &= ~(1<<5);		/* D5: P0 down  */
	if(pad & PAD_UP) pia.SWCHA &= ~(1<<4);		/* D4: P0 up    */
							/* D3: P1 right */
							/* D2: P1 left  */
							/* D1: P1 down  */
							/* D0: P1 up    */

	pia.SWCHB = 0xff;
	                     pia.SWCHB &= ~(1<<7);	/* D7: P1 difficulty 0 = amateur (B), 1 = pro (A) */
	                     pia.SWCHB &= ~(1<<6);	/* D6: P0 difficulty 0 = amateur (B), 1 = pro (A) */
							/* D5/D4: (not used) */
							/* D3: color - B/W 0 = B/W, 1 = color */
							/* D2 (not used) */
	if(pad & PAD_SELECT) pia.SWCHB &= ~(1<<1);	/* D1 game select 0 = switch pressed */
	if(pad & PAD_START ) pia.SWCHB &= ~(1<<0);	/* D0 game reset 0 = switch pressed */
}

int
vcs_stream_callback(short* wbuff, int param)
{
	//tia_mix(wbuff); TODO:
	return 0;
}

int
vcs_run(const char* fname, int sound_on)
{
	int cnt;

	//if(vcs_init(fname) != 0) {
	//Ô߁AX^bN\h
	if(stack_switch(sram_stack + sizeof sram_stack, vcs_init, (int)fname, 0)) {
		return -1;
	}

	if(sound_on) {
		//stream_play(TIABUFLEN, vcs_stream_callback, 0, 0); TODO:
	}

	cnt = 0;
	for(;;) {
		/* ͍XVB */
		input_update();

		/* CPUsB */
		CYCLE += VCS_CLOCK / 60;
		m6502cpu_run(CYCLE);
		NOW += CYCLE - m6502cpu.cycle;
		CYCLE = m6502cpu.cycle;

		/* ʍXV */
		cnt += FPS;
		if(cnt >= 60) {
			cnt -= 60;
			tia_reduce(vbuff, tbuff);
			schedule();

			/* START+SELECTŁAQ[Ivsꂽ? */
			if((joy & PAD_START) && (joy & PAD_SELECT)) {
				break;
			}
		}
	}

	if(sound_on) {
		stream_stop();
	}

	return 0;
}

/****************************************************************************
 *	
 ****************************************************************************/

int
vcs_read_tia(int addr)
{
	NOW += CYCLE - m6502cpu.cycle;
	CYCLE = m6502cpu.cycle;

	return tia_read(addr, NOW);
}

void
vcs_write_tia(int addr, int data)
{
	NOW += CYCLE - m6502cpu.cycle;
	CYCLE = m6502cpu.cycle;

	m6502cpu.cycle -= tia_write(addr, data, NOW);
}

int
vcs_read_pia(int addr)
{
	NOW += CYCLE - m6502cpu.cycle;
	CYCLE = m6502cpu.cycle;

	return pia_read(addr, NOW);
}

void
vcs_write_pia(int addr, int data)
{
	NOW += CYCLE - m6502cpu.cycle;
	CYCLE = m6502cpu.cycle;

	pia_write(addr, data, NOW);
}

